#ifndef __SWITCH_PANEL_H__
#define __SWITCH_PANEL_H__

#include "hsf.h"
#include <stdint.h>
#include <stdbool.h>
#include "gpio.h"
#include "pinctrl.h"
#include "timer.h"
#include "platform_core_rom.h"
#include "osal_task.h"
#include "hfflash.h"
#include "hfuart.h"

//====================== 硬件引脚定义 ======================
// 根据CLAUDE.md中的物理连接定义，使用标准GPIO枚举
// LED指示灯引脚 (控制按键指示灯颜色: 高电平=白灯, 低电平=黄灯)
#define LED1_GPIO           GPIO_00               // GPIO 0 - 按键1指示灯
#define LED2_GPIO           GPIO_02               // GPIO 2 - 按键2指示灯  
#define LED3_GPIO           GPIO_05               // GPIO 5 - 按键3指示灯
#define LED4_GPIO           GPIO_10               // GPIO 10 - 按键4指示灯

// 开关控制引脚 (控制实际开关: 高电平=开, 低电平=关)
#define SWITCH1_GPIO        GPIO_12               // GPIO 12 - 开关1控制
#define SWITCH2_GPIO        GPIO_13               // GPIO 13 - 开关2控制
#define SWITCH3_GPIO        GPIO_14               // GPIO 14 - 开关3控制
#define SWITCH4_GPIO        GPIO_07               // GPIO 7 - 开关4控制

// 物理按键输入引脚 (输入检测: 低电平=按下, 高电平=松开)  
#define KEY1_GPIO           GPIO_09               // GPIO 9 - 按键1输入
#define KEY2_GPIO           GPIO_04               // GPIO 4 - 按键2输入
#define KEY3_GPIO           GPIO_11               // GPIO 11 - 按键3输入
#define KEY4_GPIO           GPIO_08               // GPIO 8 - 按键4输入

// 面板背光引脚 (控制面板背光: 高电平=亮黄灯, 低电平=灭灯)
#define PANEL_LED_GPIO      GPIO_03               // GPIO 3 - 面板背光

//====================== 系统常量定义 ======================
#define SWITCH_COUNT        4                     // 开关数量
#define KEY_DEBOUNCE_MS     30                    // 按键防抖时间(毫秒)
#define KEY_LONG_PRESS_MS   10000                 // 长按时间阈值(毫秒) - 用于进入配网
#define CONFIG_TIMEOUT_MS   (10*60*1000)          // 配网超时时间(10分钟)
#define CONFIG_BLINK_MS     (3*60*1000)           // 配网前3分钟闪烁时间
#define PANEL_BLINK_MS      1000                  // 面板背光快闪时间(1秒)
#define LED_BLINK_FREQ_HZ   1                     // 配网时LED闪烁频率(1Hz)

//====================== 设备状态定义 ======================
// 开关状态
typedef enum {
    SWITCH_OFF = 0,
    SWITCH_ON = 1
} switch_state_t;

// 按键状态
typedef enum {
    KEY_RELEASED = 1,  // 高电平 - 按键松开
    KEY_PRESSED = 0    // 低电平 - 按键按下
} key_state_t;

// LED状态
typedef enum {
    LED_YELLOW = 0,    // 低电平 - 黄灯
    LED_WHITE = 1      // 高电平 - 白灯
} led_state_t;

// 面板背光状态
typedef enum {
    PANEL_LED_OFF = 0,   // 低电平 - 背光关闭
    PANEL_LED_ON = 1     // 高电平 - 背光开启(黄色)
} panel_led_state_t;

// 系统工作模式
typedef enum {
    MODE_NORMAL = 0,        // 正常工作模式
    MODE_CONFIG,            // 配网模式
    MODE_FACTORY_TEST,      // 产测模式
    MODE_UNBIND            // 设备解绑模式
} system_mode_t;

//====================== 设备状态结构 ======================
// 开关名字最大长度
#define SWITCH_NAME_MAX_LEN    32

// 单个开关的持久化状态信息（需要保存到Flash）
typedef struct {
    bool switch_on;                      // 开关状态 (true=开, false=关) - 持久化
    bool led_state;                      // LED状态 (true=白灯, false=黄灯) - 持久化
    char name[SWITCH_NAME_MAX_LEN];      // 开关名字 - 持久化
} switch_persistent_info_t;

// 单个开关的运行时状态信息（不需要持久化）
typedef struct {
    bool physical_key;      // 物理按键状态 (true=松开, false=按下)
    uint32_t press_time;    // 按键按下时间戳
    bool debounce_flag;     // 防抖标志
    bool long_press_handled; // 长按处理标志
} switch_runtime_info_t;

// 持久化设备状态（需要保存到Flash）
typedef struct {
    switch_persistent_info_t switches[SWITCH_COUNT];   // 4个开关的持久化状态
    bool master_switch;                     // 总开关状态 - 持久化
    bool panel_led;                         // 面板背光状态 - 持久化
    bool is_bound;                          // 设备绑定状态 - 持久化
    bool is_first_boot;                     // 是否第一次上电 - 持久化
    uint32_t magic;                         // 魔数标识
    uint32_t version;                       // 版本号
    uint32_t reserved[8];                   // 保留字段
} device_persistent_state_t;

// 运行时设备状态（不需要持久化，断电丢失）
typedef struct {
    switch_runtime_info_t switches[SWITCH_COUNT];  // 4个开关的运行时状态
    system_mode_t mode;                     // 系统工作模式（运行时状态）
    bool ble_mode_enabled;                  // 蓝牙模式是否启用
    uint32_t config_start_time;             // 配网开始时间
    int config_key_id;                      // 触发配网的按键ID
    bool config_led_blink_state;            // 配网LED闪烁状态
    bool factory_test_running;              // 产测是否运行中
    uint32_t last_save_time;                // 上次保存时间
    uint32_t reserved[16];                  // 保留字段
} device_runtime_state_t;


//====================== Flash存储相关 ======================
// Flash存储地址定义
#define SUPPORT_SAVE_TO_FLASH 1

// 定义Flash存储地址和大小
#define DEVICE_DATA_FLASH_SIZE    0x001000  // 使用一个页的大小
#define DEVICE_DATA_FLASH_ADDR    0x000000  // 主数据区地址
#define DEVICE_DATA_BACKUP_ADDR   (DEVICE_DATA_FLASH_ADDR + DEVICE_DATA_FLASH_SIZE)  // 备份数据区地址

// 设备数据结构
typedef struct {
    uint8_t checksum;           // 校验和
    uint32_t magic;             // 魔数，用于验证数据有效性
    uint32_t version;           // 版本号
    uint32_t data_len;          // 数据长度
    uint8_t data[0];            // 数据
} device_data_t;

#define DEVICE_DATA_MAGIC       0x4C505426  // "LPT&"的ASCII码
#define DEVICE_DATA_VERSION     1         // 数据版本号

//====================== 配网相关定义 ======================
#define FACTORY_TEST_SSID   "ShuorongSelfTest"     // 产测热点名称

//====================== 任务和定时器相关 ======================
#define TASK_STACK_SIZE     0x1000
#define TASK_PRIORITY_HIGH  OSAL_TASK_PRIORITY_HIGH
#define TASK_PRIORITY_NORM  OSAL_TASK_PRIORITY_MIDDLE
#define TASK_PRIORITY_LOW   OSAL_TASK_PRIORITY_LOW

//====================== 异步上报系统相关 ======================
// 定义服务ID的位掩码
#define REPORT_SWITCH_MASK      0x01    // switch (总开关)
#define REPORT_SWITCH1_MASK     0x02    // switch1
#define REPORT_SWITCH2_MASK     0x04    // switch2 
#define REPORT_SWITCH3_MASK     0x08    // switch3
#define REPORT_SWITCH4_MASK     0x10    // switch4
#define REPORT_ALL_MASK         0x1F    // 所有开关

// 异步上报系统相关变量
extern osal_task *g_report_task_handle;   // 异步上报任务句柄

//====================== 全局变量声明 ======================
extern device_persistent_state_t g_persistent_state;
extern device_runtime_state_t g_runtime_state;
extern timer_handle_t g_key_debounce_timer[SWITCH_COUNT];
// 配网定时器变量已移除，配网逻辑简化
extern osal_task *g_key_scan_task_handle;
extern osal_task *g_config_task_handle;
extern osal_task *g_save_task_handle;  // 异步保存任务句柄
extern osal_task *g_report_task_handle;  // 异步上报任务句柄

//====================== 函数声明 ======================

// 主程序函数
int switch_panel_main(void);

// 硬件控制函数
int key_system_init(void);
int switch_panel_gpio_init(void);
void set_switch_output(int switch_id, bool state);
void set_led_output(int led_id, led_state_t state);
void set_panel_led(panel_led_state_t state);
bool get_key_input(int key_id);

// 设备状态管理函数
int load_persistent_state(void);
int save_persistent_state(void);                // 异步保存（触发保存任务）
int save_persistent_state_sync(void);           // 同步保存（直接写入Flash）
int save_system_init(void);                     // 初始化异步保存系统
void save_system_deinit(void);                  // 清理异步保存系统
void reset_persistent_state(void);
void init_runtime_state(void);
void sync_hardware_state(void);
void fast_report_switch(int switch_id);
void fast_report_master_switch(void);
void set_device_mode(system_mode_t mode);

// 状态访问便利函数
bool get_switch_state(int switch_id);
bool get_master_switch_state(void);
bool is_device_bound(void);
bool is_first_boot(void);
system_mode_t get_device_mode(void);

// 异步上报系统管理函数
int report_system_init(void);               // 初始化异步上报系统
void report_system_deinit(void);            // 清理异步上报系统
void trigger_async_report(uint8_t report_mask);  // 触发异步上报
int async_report_task(void *arg);           // 异步上报任务

// 快速上报函数(新的异步版本)
void fast_report_switch_async(int switch_id);    // 异步上报单个开关
void fast_report_master_switch_async(void);      // 异步上报总开关
void fast_report_all_switches_async(void);       // 异步上报所有开关

// 开关名字操作函数
const char* get_switch_name(int switch_id);
int set_switch_name(int switch_id, const char* name);
void init_default_switch_names(void);

// 开关控制函数
void update_switch_state(int switch_id, bool state);
void update_master_switch(bool state);
void apply_master_switch_control(void);

// 按键处理函数
int key_scan_task(void *arg);
void handle_key_press(int key_id);
void handle_key_long_press(int key_id);

// 配网相关函数
void enter_config_mode(void);
void exit_config_mode(void);
int config_mode_task(void *arg);
void config_led_blink(void);
void panel_led_blink(void);

// 产测相关函数
void enter_factory_test_mode(void);
void factory_test_sequence(void);
bool check_factory_test_wifi(void);

// HiLink设备事件处理
void handle_device_online(void);
void handle_device_offline(void);
void handle_device_unbind(void);
void sync_cloud_state(void);

// HiLink服务处理函数
int handle_put_switch(const char* svc_id, const char* payload, unsigned int len);
int handle_get_switch(const char* svc_id, const char* in, unsigned int in_len, char** out, unsigned int* out_len);
int handle_put_switch1(const char* svc_id, const char* payload, unsigned int len);
int handle_get_switch1(const char* svc_id, const char* in, unsigned int in_len, char** out, unsigned int* out_len);
int handle_put_switch2(const char* svc_id, const char* payload, unsigned int len);
int handle_get_switch2(const char* svc_id, const char* in, unsigned int in_len, char** out, unsigned int* out_len);
int handle_put_switch3(const char* svc_id, const char* payload, unsigned int len);
int handle_get_switch3(const char* svc_id, const char* in, unsigned int in_len, char** out, unsigned int* out_len);
int handle_put_switch4(const char* svc_id, const char* payload, unsigned int len);
int handle_get_switch4(const char* svc_id, const char* in, unsigned int in_len, char** out, unsigned int* out_len);

// 调试和工具函数
void print_device_state(void);
const char* get_mode_string(system_mode_t mode);

// 定时器回调函数
void key_debounce_timer_callback(uintptr_t data);
// 配网定时器回调函数已移除，配网逻辑现在在config_mode_task中直接处理

//====================== 蓝牙控制相关函数 ======================
// 蓝牙数据处理
int switch_panel_ble_handle_custom_data(const char *buff, unsigned int len);

// 蓝牙模式管理
void switch_panel_ble_enable(void);
void switch_panel_ble_disable(void);
bool switch_panel_ble_is_enabled(void);

// 蓝牙上报
int switch_panel_ble_fast_report(const char *svc_id);

int start_hilink_ble_net_config(int32_t net_cfg_time_s);
#endif // __SWITCH_PANEL_H__