#include "switch_panel.h"
#include "hilink_device.h"
#include "ble_cfg_net_api.h"
#include "cJSON.h"
#include "securec.h"

//====================== 蓝牙控制相关常量 ======================
#define SWITCH_BLE_REPORT   "{\"data\":{\"on\":%d},\"sid\":\"%s\"}"
#define SWITCH_BLE_NAME_REPORT   "{\"data\":{\"on\":%d,\"name\":\"%s\"},\"sid\":\"%s\"}"

//====================== 蓝牙控制状态变量 ======================
static bool g_ble_mode_enabled = false;  // 蓝牙控制模式是否启用

//====================== 蓝牙上报函数 ======================

// 通过蓝牙上报总开关状态
static void ble_report_master_switch(void) {
    char buff[128] = {0};
    int ret = snprintf_s(buff, sizeof(buff), sizeof(buff) - 1, 
                        SWITCH_BLE_REPORT, 
                        g_persistent_state.master_switch ? 1 : 0, 
                        "switch");
    if (ret <= 0) {
        e_printf("[BLE] 总开关状态格式化失败: %d\r\n", ret);
        return;
    }
    
    unsigned int buffLen = strlen(buff);
    ret = BLE_SendCustomData(CUSTOM_SEC_DATA, (const unsigned char *)buff, buffLen);
    e_printf("[BLE] 上报总开关状态: %s, 结果: %d\r\n", buff, ret);
}

// 通过蓝牙上报单个开关状态
static void ble_report_switch_state(int switch_id) {
    if (switch_id < 0 || switch_id >= SWITCH_COUNT) {
        e_printf("[BLE] 无效的开关ID: %d\r\n", switch_id);
        return;
    }
    
    char buff[128] = {0};
    char svc_id[16] = {0};
    char switch_name[16] = {0};
    
    snprintf_s(svc_id, sizeof(svc_id), sizeof(svc_id) - 1, "switch%d", switch_id + 1);
    snprintf_s(switch_name, sizeof(switch_name), sizeof(switch_name) - 1, "开关%d", switch_id + 1);
    
    int ret = snprintf_s(buff, sizeof(buff), sizeof(buff) - 1, 
                        SWITCH_BLE_NAME_REPORT,
                        g_persistent_state.switches[switch_id].switch_on ? 1 : 0,
                        switch_name,
                        svc_id);
    if (ret <= 0) {
        e_printf("[BLE] 开关%d状态格式化失败: %d\r\n", switch_id + 1, ret);
        return;
    }
    
    unsigned int buffLen = strlen(buff);
    ret = BLE_SendCustomData(CUSTOM_SEC_DATA, (const unsigned char *)buff, buffLen);
    e_printf("[BLE] 上报开关%d状态: %s, 结果: %d\r\n", switch_id + 1, buff, ret);
}

// 通过蓝牙上报所有开关状态
static void ble_report_all_switches(void) {
    e_printf("[BLE] 开始上报所有开关状态\r\n");
    
    // 上报总开关状态
    ble_report_master_switch();
    
    // 上报所有子开关状态
    for (int i = 0; i < SWITCH_COUNT; i++) {
        ble_report_switch_state(i);
        osal_msleep(50);  // 避免发送过快
    }
    
    e_printf("[BLE] 所有开关状态上报完成\r\n");
}

//====================== 蓝牙接收数据处理 ======================

// 处理总开关控制指令
static int ble_handle_master_switch(cJSON *dataItem) {
    cJSON *onItem = cJSON_GetObjectItem(dataItem, "on");
    if (onItem == NULL || !cJSON_IsNumber(onItem)) {
        e_printf("[BLE] 总开关控制指令格式错误\r\n");
        return -1;
    }
    
    bool new_state = (onItem->valueint != 0);
    e_printf("[BLE] 接收到总开关控制指令: %s\r\n", new_state ? "开" : "关");
    
    // 调用现有的总开关控制函数
    update_master_switch(new_state);
    
    // 通过蓝牙上报状态确认
    ble_report_master_switch();
    
    return 0;
}

// 处理子开关控制指令
static int ble_handle_switch_control(const char *svc_id, cJSON *dataItem) {
    // 解析开关ID (switch1 -> 0, switch2 -> 1, etc.)
    int switch_id = -1;
    if (strcmp(svc_id, "switch1") == 0) {
        switch_id = 0;
    } else if (strcmp(svc_id, "switch2") == 0) {
        switch_id = 1;
    } else if (strcmp(svc_id, "switch3") == 0) {
        switch_id = 2;
    } else if (strcmp(svc_id, "switch4") == 0) {
        switch_id = 3;
    } else {
        e_printf("[BLE] 未知的开关ID: %s\r\n", svc_id);
        return -1;
    }
    
    cJSON *onItem = cJSON_GetObjectItem(dataItem, "on");
    if (onItem == NULL || !cJSON_IsNumber(onItem)) {
        e_printf("[BLE] 开关%d控制指令格式错误\r\n", switch_id + 1);
        return -1;
    }
    
    bool new_state = (onItem->valueint != 0);
    e_printf("[BLE] 接收到开关%d控制指令: %s\r\n", switch_id + 1, new_state ? "开" : "关");
    
    // 调用现有的开关控制函数
    update_switch_state(switch_id, new_state);
    
    // 通过蓝牙上报状态确认
    ble_report_switch_state(switch_id);
    
    return 0;
}

// 处理状态查询指令
static int ble_handle_status_query(const char *svc_id) {
    e_printf("[BLE] 接收到状态查询: %s\r\n", svc_id);
    
    if (strcmp(svc_id, "switch") == 0) {
        // 查询总开关状态
        ble_report_master_switch();
    } else if (strncmp(svc_id, "switch", 6) == 0 && strlen(svc_id) == 7) {
        // 查询子开关状态 (switch1, switch2, etc.)
        int switch_id = svc_id[6] - '1';  // '1' -> 0, '2' -> 1, etc.
        if (switch_id >= 0 && switch_id < SWITCH_COUNT) {
            ble_report_switch_state(switch_id);
        } else {
            e_printf("[BLE] 无效的开关查询ID: %s\r\n", svc_id);
            return -1;
        }
    } else {
        e_printf("[BLE] 未知的查询ID: %s\r\n", svc_id);
        return -1;
    }
    
    return 0;
}

//====================== 蓝牙数据处理主函数 ======================

// 处理蓝牙接收的自定义数据
int switch_panel_ble_handle_custom_data(const char *buff, unsigned int len) {
    if (buff == NULL || len == 0) {
        e_printf("[BLE] 接收数据为空\r\n");
        return -1;
    }
    
    e_printf("[BLE] 接收到数据: %s (长度: %u)\r\n", buff, len);
    
    // 处理空JSON请求 - 上报全量状态
    if (strcmp(buff, "{}") == 0) {
        e_printf("[BLE] 收到全量状态请求\r\n");
        ble_report_all_switches();
        return 0;
    }
    
    // 解析JSON数据
    cJSON *json = cJSON_Parse(buff);
    if (json == NULL) {
        e_printf("[BLE] JSON解析失败\r\n");
        return -1;
    }
    
    int ret = -1;
    
    do {
        // 获取服务ID
        cJSON *sidItem = cJSON_GetObjectItem(json, "sid");
        if (sidItem == NULL || !cJSON_IsString(sidItem) || sidItem->valuestring == NULL) {
            e_printf("[BLE] 缺少或无效的服务ID\r\n");
            break;
        }
        
        const char *svc_id = sidItem->valuestring;
        cJSON *dataItem = cJSON_GetObjectItem(json, "data");
        
        // 处理特殊服务ID
        if (strcmp(svc_id, "allservices") == 0) {
            // H5连接时的状态同步请求
            e_printf("[BLE] H5连接，同步所有状态\r\n");
            ble_report_all_switches();
            ret = 0;
            break;
        } else if (strcmp(svc_id, "currentTime") == 0) {
            // 时间同步（可选实现）
            e_printf("[BLE] 接收到时间同步请求\r\n");
            ret = 0;
            break;
        }
        
        // 如果没有data字段，表示状态查询
        if (dataItem == NULL) {
            ret = ble_handle_status_query(svc_id);
            break;
        }
        
        // 处理控制指令
        if (strcmp(svc_id, "switch") == 0) {
            // 总开关控制
            ret = ble_handle_master_switch(dataItem);
        } else if (strncmp(svc_id, "switch", 6) == 0 && strlen(svc_id) == 7) {
            // 子开关控制 (switch1, switch2, switch3, switch4)
            ret = ble_handle_switch_control(svc_id, dataItem);
        } else {
            e_printf("[BLE] 未知的服务ID: %s\r\n", svc_id);
        }
        
    } while (0);
    
    cJSON_Delete(json);
    return ret;
}

//====================== 蓝牙模式管理 ======================

// 启用蓝牙控制模式
void switch_panel_ble_enable(void) {
    g_ble_mode_enabled = true;
    e_printf("[BLE] 蓝牙控制模式已启用\r\n");
}

// 禁用蓝牙控制模式
void switch_panel_ble_disable(void) {
    g_ble_mode_enabled = false;
    e_printf("[BLE] 蓝牙控制模式已禁用\r\n");
}

// 检查蓝牙控制模式是否启用
bool switch_panel_ble_is_enabled(void) {
    return g_ble_mode_enabled;
}

//====================== 蓝牙上报接口 ======================

// 蓝牙模式下的快速上报函数
int switch_panel_ble_fast_report(const char *svc_id) {
    if (!g_ble_mode_enabled) {
        return -1;  // 蓝牙模式未启用
    }
    
    if (svc_id == NULL) {
        e_printf("[BLE] 服务ID为空\r\n");
        return -1;
    }
    
    e_printf("[BLE] 快速上报服务: %s\r\n", svc_id);
    
    if (strcmp(svc_id, "switch") == 0) {
        ble_report_master_switch();
    } else if (strcmp(svc_id, "switch1") == 0) {
        ble_report_switch_state(0);
    } else if (strcmp(svc_id, "switch2") == 0) {
        ble_report_switch_state(1);
    } else if (strcmp(svc_id, "switch3") == 0) {
        ble_report_switch_state(2);
    } else if (strcmp(svc_id, "switch4") == 0) {
        ble_report_switch_state(3);
    } else {
        e_printf("[BLE] 未知的服务ID: %s\r\n", svc_id);
        return -1;
    }
    
    return 0;
}