#include "stdint.h"
#include "hsf.h"
#include "hfsys.h"
#include "hfgpio.h"
#include "switch_panel.h"
#include "hilink.h"
#include "hsf.h"
#include "hfconfig.h"
#include "pinctrl.h"
#include "timer.h"
#include "soc_osal.h"
#include "systick.h"
#include "wifi_hotspot.h"
#include "wifi_hotspot_config.h"
#include "lwip/netifapi.h"

//====================== 配网相关变量 ======================
int g_config_key_id = -1;  // 触发配网的按键 - 移除static使其可被外部访问

//====================== 产测相关常量定义 ======================
#define FACTORY_TEST_RSSI_THRESHOLD (-70)        // WiFi信号强度阈值 -70dBm
#define WIFI_SCAN_AP_LIMIT          64           // 最大扫描AP数量
#define WIFI_MAX_SSID_LEN           33           // SSID最大长度
#define WIFI_MAC_LEN                6            // MAC地址长度
#define WIFI_STA_SAMPLE_LOG         "[FACTORY]"  // 日志前缀

//====================== 产测WiFi状态枚举 ======================
enum {
    WIFI_STA_SAMPLE_INIT = 0,       // 初始态
    WIFI_STA_SAMPLE_SCANING,        // 扫描中
    WIFI_STA_SAMPLE_SCAN_DONE,      // 扫描完成
    WIFI_STA_SAMPLE_FOUND_TARGET,   // 匹配到目标AP
} wifi_state_enum;

//====================== 产测相关变量 ======================
static bool g_factory_test_running = false;
static int g_factory_test_step = 0;
static uint32_t g_factory_test_start_time = 0;
static uint8_t g_wifi_state = WIFI_STA_SAMPLE_INIT;

//====================== WiFi事件回调函数 ======================
static void wifi_scan_state_changed(int32_t state, int32_t size) {
    UNUSED(state);
    UNUSED(size);
    e_printf("%s WiFi扫描完成!\r\n", WIFI_STA_SAMPLE_LOG);
    g_wifi_state = WIFI_STA_SAMPLE_SCAN_DONE;
}

static void wifi_connection_changed(int32_t state, const wifi_linked_info_stru *info, int32_t reason_code) {
    UNUSED(info);
    UNUSED(reason_code);
    // 产测模式下不需要连接，只需要扫描
}

static wifi_event_stru wifi_event_cb = {
    .wifi_event_connection_changed = wifi_connection_changed,
    .wifi_event_scan_state_changed = wifi_scan_state_changed,
};

//====================== WiFi扫描和产测函数 ======================

// 匹配目标AP并获取信号强度
static int32_t get_match_network_rssi(const char* target_ssid, int32_t* rssi) {
    int32_t ret;
    uint32_t num = WIFI_SCAN_AP_LIMIT;
    bool find_ap = false;
    uint8_t bss_index;
    
    // 获取扫描结果
    uint32_t scan_len = sizeof(wifi_scan_info_stru) * WIFI_SCAN_AP_LIMIT;
    wifi_scan_info_stru *result = osal_kmalloc(scan_len, OSAL_GFP_ATOMIC);
    if (result == NULL) {
        e_printf("%s 内存分配失败\r\n", WIFI_STA_SAMPLE_LOG);
        return -1;
    }
    
    memset_s(result, scan_len, 0, scan_len);
    ret = wifi_sta_get_scan_info(result, &num);
    if (ret != 0) {
        e_printf("%s 获取扫描信息失败: %d\r\n", WIFI_STA_SAMPLE_LOG, ret);
        osal_kfree(result);
        return -1;
    }
    
    // 筛选扫描到的WiFi网络，查找目标AP
    for (bss_index = 0; bss_index < num; bss_index++) {
        if (strlen(target_ssid) == strlen(result[bss_index].ssid)) {
            if (memcmp(target_ssid, result[bss_index].ssid, strlen(target_ssid)) == 0) {
                find_ap = true;
                *rssi = result[bss_index].rssi;
                e_printf("%s 找到目标AP: %s, 信号强度: %ddBm\r\n", 
                       WIFI_STA_SAMPLE_LOG, target_ssid, *rssi);
                break;
            }
        }
    }
    
    osal_kfree(result);
    
    if (!find_ap) {
        e_printf("%s 未找到目标AP: %s\r\n", WIFI_STA_SAMPLE_LOG, target_ssid);
        return -1;
    }
    
    return 0;
}

// WiFi扫描并检查信号强度
static int32_t wifi_scan_and_check(const char* target_ssid, int32_t rssi_threshold) {
    int32_t rssi = 0;
    g_wifi_state = WIFI_STA_SAMPLE_INIT;
    
    do {
        osal_msleep(10);
        
        if (g_wifi_state == WIFI_STA_SAMPLE_INIT) {
            e_printf("%s 开始WiFi扫描...\r\n", WIFI_STA_SAMPLE_LOG);
            g_wifi_state = WIFI_STA_SAMPLE_SCANING;
            
            // 启动WiFi扫描
            if (wifi_sta_scan() != 0) {
                e_printf("%s WiFi扫描启动失败\r\n", WIFI_STA_SAMPLE_LOG);
                g_wifi_state = WIFI_STA_SAMPLE_INIT;
                continue;
            }
        } else if (g_wifi_state == WIFI_STA_SAMPLE_SCAN_DONE) {
            // 获取目标网络的信号强度
            if (get_match_network_rssi(target_ssid, &rssi) != 0) {
                e_printf("%s 未找到目标AP: %s\r\n", WIFI_STA_SAMPLE_LOG, target_ssid);
                return -1;
            }
            break;
        }
    } while (1);
    
    // 检查信号强度阈值
    if (rssi_threshold == 0) {
        // 只检查是否找到目标AP，不检查信号强度
        return 0;
    }
    
    e_printf("%s 信号强度测试: 期望 >= %ddBm, 实际 %ddBm\r\n", 
           WIFI_STA_SAMPLE_LOG, rssi_threshold, rssi);
           
    if (rssi >= rssi_threshold) {
        return 0;  // 测试通过
    }
    
    return -1;  // 信号强度不足
}

// 初始化WiFi产测环境
static int factory_test_wifi_init(void) {
    // 注册WiFi事件回调
    if (wifi_register_event_cb(&wifi_event_cb) != 0) {
        e_printf("%s WiFi事件回调注册失败\r\n", WIFI_STA_SAMPLE_LOG);
        return -1;
    }
    
    // 等待WiFi初始化完成
    int timeout = 100;  // 10秒超时
    while (wifi_is_wifi_inited() == 0 && timeout-- > 0) {
        msleep(100);
    }
    
    if (timeout <= 0) {
        e_printf("%s WiFi初始化超时\r\n", WIFI_STA_SAMPLE_LOG);
        goto lab_err;
    }
    
    // 创建STA接口
    if (wifi_sta_enable() != 0) {
        e_printf("%s WiFi STA启动失败\r\n", WIFI_STA_SAMPLE_LOG);
        goto lab_err;
    }
    
    e_printf("%s WiFi产测环境初始化完成\r\n", WIFI_STA_SAMPLE_LOG);
    return 0;
lab_err:
    wifi_unregister_event_cb(&wifi_event_cb);
    return -1;
}

// 进入配网模式
void enter_config_mode(void) {
    if (g_runtime_state.mode == MODE_CONFIG) {
        e_printf("[CONFIG] 已在配网模式中\r\n");
        return;
    }
    
    e_printf("[CONFIG] 进入配网模式\r\n");
    start_hilink_ble_net_config(CONFIG_TIMEOUT_MS/1000);
    
    // 更新设备模式
    set_device_mode(MODE_CONFIG);

    // 面板背光快闪1秒表示进入配网模式
    panel_led_blink();
    
    // 创建配网任务
    if (!g_config_task_handle) {
        g_config_task_handle = osal_kthread_create((osal_kthread_handler)config_mode_task, 
                                                  NULL, 
                                                  "config_task", 
                                                  TASK_STACK_SIZE);
        if (!g_config_task_handle) {
            e_printf("[CONFIG] 创建配网任务失败\r\n");
            return;
        }
        
        // 设置任务优先级
        osal_kthread_set_priority(g_config_task_handle, TASK_PRIORITY_NORM);
    }
    
    e_printf("[CONFIG] 配网模式已开启，超时时间: %d分钟\r\n", CONFIG_TIMEOUT_MS / 60000);
}

// 退出配网模式
void exit_config_mode(void) {
    if (g_runtime_state.mode != MODE_CONFIG) {
        return;
    }
    
    e_printf("[CONFIG] 退出配网模式\r\n");
    
    // 终止配网任务
    if (g_config_task_handle) {
        osal_kthread_destroy(g_config_task_handle, 1);
        g_config_task_handle = NULL;
    }
    
    // 恢复正常LED状态
    for (int i = 0; i < SWITCH_COUNT; i++) {
        set_led_output(i, g_persistent_state.switches[i].led_state ? LED_WHITE : LED_YELLOW);
    }
    
    // 恢复面板背光状态
    set_panel_led(g_persistent_state.panel_led ? PANEL_LED_ON : PANEL_LED_OFF);
    
    // 更新设备模式
    set_device_mode(MODE_NORMAL);
    
    // 重置配网相关变量
    g_config_key_id = -1;
    
    e_printf("[CONFIG] 已退出配网模式\r\n");
}

// 配网模式任务
int config_mode_task(void *arg) {
    (void)arg;
    
    e_printf("[CONFIG] 配网任务开始\r\n");
        
    // 记录配网开始时间
    g_runtime_state.config_start_time = hfsys_get_time();
    
    // 只有在首次启动时才检查产测热点
    if (g_persistent_state.is_first_boot && check_factory_test_wifi()) {
        e_printf("[CONFIG] 检测到产测热点且为首次启动，进入产测模式\r\n");
        exit_config_mode();
        enter_factory_test_mode();
        return 0;
    }
    
    uint32_t config_start_time = g_runtime_state.config_start_time;
    bool led_blink_state = false;
    uint32_t last_blink_time = 0;
    const uint32_t blink_interval = 500; // 500ms闪烁间隔 (1Hz)
    
    while (g_runtime_state.mode == MODE_CONFIG && !osal_kthread_should_stop()) {
        uint32_t current_time = hfsys_get_time();
        uint32_t elapsed_time = current_time - config_start_time;
        
        // 检查是否超时 (10分钟)
        if (elapsed_time >= CONFIG_TIMEOUT_MS) {
            e_printf("[CONFIG] 配网超时，退出配网模式\r\n");
            break;
        }
        
        // LED控制逻辑
        if (elapsed_time < CONFIG_BLINK_MS) {  // 前3分钟闪烁
            // 检查是否需要切换LED状态
            if (current_time - last_blink_time >= blink_interval) {
                led_blink_state = !led_blink_state;
                last_blink_time = current_time;
                
                // 只有触发配网的按键LED闪烁，其他LED保持黄灯常亮
                for (int i = 0; i < SWITCH_COUNT; i++) {
                    if (i == g_config_key_id) {
                        // 触发配网的按键LED闪烁
                        set_led_output(i, led_blink_state ? LED_WHITE : LED_YELLOW);
                    } else {
                        // 其他按键LED保持黄灯常亮
                        set_led_output(i, LED_YELLOW);
                    }
                }
            }
        } else {  // 后7分钟常亮
            // 所有LED保持黄灯常亮
            for (int i = 0; i < SWITCH_COUNT; i++) {
                set_led_output(i, LED_YELLOW);
            }
        }
        
        // 每100ms检查一次
        osal_msleep(100);
    }

    e_printf("[CONFIG] 配网任务结束\r\n");
    g_config_task_handle = NULL;
    // 退出配网模式
    exit_config_mode();
    return 0;
}

// 面板背光快闪
void panel_led_blink(void) {
    // 快闪1秒
    for (int i = 0; i < 10; i++) {
        set_panel_led(PANEL_LED_ON);
        osal_msleep(50);
        set_panel_led(PANEL_LED_OFF);
        osal_msleep(50);
    }
    
    // 恢复原状态
    set_panel_led(g_persistent_state.panel_led ? PANEL_LED_ON : PANEL_LED_OFF);
}

//====================== 产测模式函数 ======================

// 进入产测模式
void enter_factory_test_mode(void) {
    e_printf("[FACTORY] 进入产测模式\r\n");
    
    // 更新设备模式
    set_device_mode(MODE_FACTORY_TEST);
    
    // 初始化产测变量
    g_factory_test_running = true;
    g_factory_test_step = 0;
    g_factory_test_start_time = hfsys_get_time();
    
    // 开始产测序列
    factory_test_sequence();
}

// 产测序列
void factory_test_sequence(void) {
    e_printf("[FACTORY] 开始产测序列\r\n");
    
    // 步骤1: 开始指示 - 交替闪烁白黄灯500ms
    for (int i = 0; i < 6; i++) {
        for (int j = 0; j < SWITCH_COUNT; j++) {
            set_led_output(j, (i % 2) ? LED_WHITE : LED_YELLOW);
        }
        osal_msleep(500);
    }
    
    // 步骤2-5: 单个开关测试 (开1->4, 每个1.5秒)
    for (int switch_id = 0; switch_id < SWITCH_COUNT; switch_id++) {
        e_printf("[FACTORY] 测试开关%d\r\n", switch_id + 1);
        
        // 所有开关关闭，所有LED黄灯
        for (int i = 0; i < SWITCH_COUNT; i++) {
            set_switch_output(i, false);
            set_led_output(i, LED_YELLOW);
        }
        
        // 当前测试的开关开启且LED白灯
        set_switch_output(switch_id, true);
        set_led_output(switch_id, LED_WHITE);
        
        // 保持1.5秒
        osal_msleep(1500);
    }
    
    // 步骤6: 全关 (1.5秒)
    e_printf("[FACTORY] 测试全关\r\n");
    for (int i = 0; i < SWITCH_COUNT; i++) {
        set_switch_output(i, false);
        set_led_output(i, LED_YELLOW);
    }
    osal_msleep(1500);
    
    // 步骤7: 全开 (1.5秒)
    e_printf("[FACTORY] 测试全开\r\n");
    for (int i = 0; i < SWITCH_COUNT; i++) {
        set_switch_output(i, true);
        set_led_output(i, LED_WHITE);
    }
    osal_msleep(1500);
    
    // 步骤8: 再次全关
    e_printf("[FACTORY] 结束测试 - 全关\r\n");
    for (int i = 0; i < SWITCH_COUNT; i++) {
        set_switch_output(i, false);
        set_led_output(i, LED_YELLOW);
    }
    
    // TODO: WiFi信号强度校验
    bool wifi_test_pass = check_factory_test_wifi();
    
    // 显示最终结果
    if (wifi_test_pass) {
        e_printf("[FACTORY] 产测全部通过\r\n");
        // 所有开关开启表示通过
        for (int i = 0; i < SWITCH_COUNT; i++) {
            set_switch_output(i, true);
            set_led_output(i, LED_WHITE);
        }
    } else {
        e_printf("[FACTORY] 产测失败\r\n");
        // 所有开关关闭表示失败
        for (int i = 0; i < SWITCH_COUNT; i++) {
            set_switch_output(i, false);
            set_led_output(i, LED_YELLOW);
        }
    }
    
    // 产测完成，返回正常模式
    g_factory_test_running = false;
    set_device_mode(MODE_NORMAL);
    
    // 恢复设备默认状态
    sync_hardware_state();
    
    e_printf("[FACTORY] 产测序列完成\r\n");
}

// 检查产测热点
bool check_factory_test_wifi(void) {
    // 使用WiFi扫描功能检测产测热点 "ShuorongSelfTest"
    // 并校验信号强度 >= -70dBm
    
    static bool wifi_initialized = false;
    
    // 初始化WiFi环境（只初始化一次）
    if (!wifi_initialized) {
        if (factory_test_wifi_init() != 0) {
            e_printf("%s WiFi环境初始化失败\r\n", WIFI_STA_SAMPLE_LOG);
            return false;
        }
        wifi_initialized = true;
        
        // 等待WiFi稳定
        // osal_msleep(1000);
    }
    
    // 扫描并检查目标热点
    if (wifi_scan_and_check(FACTORY_TEST_SSID, FACTORY_TEST_RSSI_THRESHOLD) == 0) {
        e_printf("%s 产测热点检测通过: %s, 信号强度满足要求\r\n", 
               WIFI_STA_SAMPLE_LOG, FACTORY_TEST_SSID);
        return true;
    }
    
    return false;  // 未找到合格的产测热点
}

