#include <stdbool.h>
#include "hsf.h"
#include "hfgpio.h"
#include "hilink.h"
#include "securec.h"
#include "cJSON.h"
#include "switch_panel.h"

static int handle_get_switch_common(int switch_id, const char* svc_id, 
                                   const char* in, unsigned int in_len,
                                   char** out, unsigned int* out_len);
static int handle_put_switch_common(int switch_id, const char* svc_id, 
                                   const char* payload, unsigned int len);
//====================== HiLink 设备事件处理 ======================

// 处理设备上线事件
void handle_device_online(void) {
    e_printf("设备上线\r\n");
    
    // 更新设备绑定状态
    g_persistent_state.is_bound = true;
    
    // 如果是首次启动，标记为非首次启动（首次绑定完成）
    if (g_persistent_state.is_first_boot) {
        g_persistent_state.is_first_boot = false;
        e_printf("首次绑定完成，标记为非首次启动\r\n");
    }
    
    // 退出配网模式
    if (g_runtime_state.mode == MODE_CONFIG) {
        exit_config_mode();
    }
    
    // 设备上线时禁用蓝牙模式，启用云端模式
    extern void switch_panel_ble_disable(void);
    switch_panel_ble_disable();
    
    // 同步所有状态到云端
    sync_cloud_state();
    
    // 保存状态
    save_persistent_state();
}

// 处理设备下线事件
void handle_device_offline(void) {
    e_printf("设备下线\r\n");
    
    // 设备下线时启用蓝牙模式，支持本地控制
    extern void switch_panel_ble_enable(void);
    switch_panel_ble_enable();
    
    // 设备下线时保持现有状态，不做特殊处理
}

// 处理设备解绑事件
void handle_device_unbind(void) {
    e_printf("设备解绑\r\n");
    
    // 更新设备绑定状态
    g_persistent_state.is_bound = false;
    g_runtime_state.mode = MODE_UNBIND;
    
    // 重置为出厂默认状态
    reset_persistent_state();
    g_persistent_state.is_bound = false;  // 保持未绑定状态
    
    // 同步硬件状态
    sync_hardware_state();
    
    // 保存状态
    save_persistent_state();
    
    e_printf("设备已重置为出厂默认状态\r\n");
}

// 同步所有状态到云端
void sync_cloud_state(void) {
    e_printf("开始同步状态到云端\r\n");
    
    // 使用批量异步上报所有开关状态
    fast_report_all_switches_async();
    
    e_printf("状态同步完成\r\n");
}

//====================== HiLink 服务处理函数 ======================

// 处理总开关PUT命令
int handle_put_switch(const char* svc_id, const char* payload, unsigned int len) {
    if (!svc_id || !payload) {
        e_printf("handle_put_switch 参数无效\r\n");
        return -1;
    }
    
    e_printf("收到总开关控制命令: %s\r\n", payload);
    
    cJSON* json = cJSON_Parse(payload);
    if (!json) {
        e_printf("JSON解析失败\r\n");
        return -1;
    }
    
    cJSON* on_item = cJSON_GetObjectItem(json, "on");
    update_master_switch(cJSON_GetNumberValue(on_item));
    
    cJSON_Delete(json);
    return 0;
}

// 处理总开关GET命令
int handle_get_switch(const char* svc_id, const char* in, unsigned int in_len, 
                     char** out, unsigned int* out_len) {
    if (!out || !out_len) {
        return -1;
    }
    
    *out_len = 32;
    *out = (char*)malloc(*out_len);
    if (!*out) {
        return -1;
    }
    
    *out_len = snprintf(*out, *out_len, "{\"on\":%d}", 
                       g_persistent_state.master_switch ? 1 : 0);
    
    e_printf("返回总开关状态: %s\r\n", *out);
    return 0;
}

// 处理开关1 PUT命令
int handle_put_switch1(const char* svc_id, const char* payload, unsigned int len) {
    return handle_put_switch_common(0, svc_id, payload, len);
}

// 处理开关1 GET命令
int handle_get_switch1(const char* svc_id, const char* in, unsigned int in_len, 
                      char** out, unsigned int* out_len) {
    return handle_get_switch_common(0, svc_id, in, in_len, out, out_len);
}

// 处理开关2 PUT命令
int handle_put_switch2(const char* svc_id, const char* payload, unsigned int len) {
    return handle_put_switch_common(1, svc_id, payload, len);
}

// 处理开关2 GET命令
int handle_get_switch2(const char* svc_id, const char* in, unsigned int in_len, 
                      char** out, unsigned int* out_len) {
    return handle_get_switch_common(1, svc_id, in, in_len, out, out_len);
}

// 处理开关3 PUT命令
int handle_put_switch3(const char* svc_id, const char* payload, unsigned int len) {
    return handle_put_switch_common(2, svc_id, payload, len);
}

// 处理开关3 GET命令
int handle_get_switch3(const char* svc_id, const char* in, unsigned int in_len, 
                      char** out, unsigned int* out_len) {
    return handle_get_switch_common(2, svc_id, in, in_len, out, out_len);
}

// 处理开关4 PUT命令
int handle_put_switch4(const char* svc_id, const char* payload, unsigned int len) {
    return handle_put_switch_common(3, svc_id, payload, len);
}

// 处理开关4 GET命令
int handle_get_switch4(const char* svc_id, const char* in, unsigned int in_len, 
                      char** out, unsigned int* out_len) {
    return handle_get_switch_common(3, svc_id, in, in_len, out, out_len);
}

//====================== 子开关通用处理函数 ======================

// 子开关PUT命令通用处理函数
static int handle_put_switch_common(int switch_id, const char* svc_id, 
                                   const char* payload, unsigned int len) {
    if (switch_id < 0 || switch_id >= SWITCH_COUNT || !svc_id || !payload) {
        e_printf("handle_put_switch%d 参数无效\r\n", switch_id + 1);
        return -1;
    }
    
    e_printf("收到开关%d控制命令: %s\r\n", switch_id + 1, payload);
    
    cJSON* json = cJSON_Parse(payload);
    if (!json) {
        e_printf("JSON解析失败\r\n");
        return -1;
    }
    
    cJSON* on_item = cJSON_GetObjectItem(json, "on");
    if (on_item) {
        update_switch_state(switch_id, cJSON_GetNumberValue(on_item));    
    }
    cJSON* name_item = cJSON_GetObjectItem(json, "name");
    if (name_item) {
        set_switch_name(switch_id, cJSON_GetStringValue(name_item));
    }
    
    cJSON_Delete(json);
    return 0;
}

// 子开关GET命令通用处理函数
static int handle_get_switch_common(int switch_id, const char* svc_id, 
                                   const char* in, unsigned int in_len,
                                   char** out, unsigned int* out_len) {
    if (switch_id < 0 || switch_id >= SWITCH_COUNT || !out || !out_len) {
        return -1;
    }
    
    *out_len = 64;
    *out = (char*)malloc(*out_len);
    if (!*out) {
        return -1;
    }
    
    *out_len = snprintf(*out, *out_len, 
                       "{\"on\":%d,\"name\":\"%s\"}",
                       g_persistent_state.switches[switch_id].switch_on ? 1 : 0,
                       g_persistent_state.switches[switch_id].name);
    
    e_printf("返回开关%d状态: %s\r\n", switch_id + 1, *out);
    return 0;
}

//====================== HiLink 事件扩展函数 ======================

// 检查设备是否在线
bool is_device_online(void) {
    return g_persistent_state.is_bound;
}

// 获取设备当前模式
system_mode_t get_current_mode(void) {
    return g_runtime_state.mode;
}