#include "switch_panel.h"
#include "hilink.h"

//====================== 按键检测相关变量 ======================
static bool g_key_states[SWITCH_COUNT] = {true, true, true, true};  // 当前按键状态

//====================== 开关控制函数 ======================

// 更新单个开关状态
void update_switch_state(int switch_id, bool state) {
    if (switch_id < 0 || switch_id >= SWITCH_COUNT) {
        e_printf("无效的开关ID: %d\r\n", switch_id);
        return;
    }
    
    // 检查总开关是否允许操作
    if (!g_persistent_state.master_switch && state) {
        e_printf("总开关关闭，不允许开启开关%d\r\n", switch_id + 1);
        return;
    }
    
    // 更新开关状态
    if (g_persistent_state.switches[switch_id].switch_on != state) {
        g_persistent_state.switches[switch_id].switch_on = state;
        
        // 更新LED指示灯(开关开启时LED白灯，关闭时LED黄灯)
        g_persistent_state.switches[switch_id].led_state = state;
        
        // 同步硬件状态
        set_switch_output(switch_id, state);
        set_led_output(switch_id, state ? LED_WHITE : LED_YELLOW);
        
        e_printf("开关%d 状态更新: %s\r\n", 
               switch_id + 1, state ? "开" : "关");
        
        // 立即保存状态
        save_persistent_state();
        
        // 上报状态给云端
        if (g_persistent_state.is_bound) {
            fast_report_switch(switch_id);
        }
    }
}

// 更新总开关状态
void update_master_switch(bool state) {
    if (g_persistent_state.master_switch != state) {
        g_persistent_state.master_switch = state;
        
        e_printf("总开关状态更新: %s\r\n", state ? "开" : "关");
        
        // 应用总开关控制逻辑
        apply_master_switch_control();
        
        // 立即保存状态
        save_persistent_state();
        
        // 上报状态给云端
        if (g_persistent_state.is_bound) {
            fast_report_master_switch();
        }
    }
}

// 应用总开关控制逻辑
void apply_master_switch_control(void) {
    
    if (!g_persistent_state.master_switch) {
        // 总开关关闭时，强制关闭所有子开关
        e_printf("总开关关闭，强制关闭所有子开关\r\n");
        
        for (int i = 0; i < SWITCH_COUNT; i++) {
            // 如果子开关之前是开着的，需要同步状态
            if (g_persistent_state.switches[i].switch_on) {
                g_persistent_state.switches[i].switch_on = false;
                g_persistent_state.switches[i].led_state = false;
                e_printf("强制关闭子开关%d\r\n", i + 1);
            }
            
            // 更新硬件状态
            set_switch_output(i, false);
            set_led_output(i, LED_YELLOW);
        }
    } else {
        // 总开关开启时，恢复各开关的原状态（不改变子开关状态）
        // e_printf("总开关开启，恢复各子开关原有状态\r\n");
        
        // for (int i = 0; i < SWITCH_COUNT; i++) {
        //     // 硬件状态跟随子开关的实际状态
        //     set_switch_output(i, g_persistent_state.switches[i].switch_on);
        //     set_led_output(i, g_persistent_state.switches[i].led_state ? LED_WHITE : LED_YELLOW);
        // }
    }
    // 立即保存状态
    save_persistent_state();
    
    // 同步所有子开关状态到云端
    if (g_persistent_state.is_bound) {
        fast_report_all_switches_async();
    }
    
    e_printf("所有子开关状态已同步\r\n");
}

//====================== 按键检测与处理 ======================

// 按键扫描任务（去除定时器防抖，在线程内直接处理防抖）
int key_scan_task(void *arg) {
    (void)arg;
    
    e_printf("[KEY] 按键扫描任务开始\r\n");
    
    // 按键状态记录
    bool stable_states[SWITCH_COUNT] = {true, true, true, true};    // 稳定状态
    bool raw_states[SWITCH_COUNT] = {true, true, true, true};       // 原始状态
    uint32_t last_change_time[SWITCH_COUNT] = {0};                  // 上次状态变化时间
    uint32_t press_start_time[SWITCH_COUNT] = {0};                  // 按键按下开始时间
    bool is_long_press_handled[SWITCH_COUNT] = {false};             // 长按处理标志
    
    // 初始化稳定状态
    for (int i = 0; i < SWITCH_COUNT; i++) {
        stable_states[i] = get_key_input(i);
        raw_states[i] = stable_states[i];
        g_key_states[i] = stable_states[i];
        g_runtime_state.switches[i].physical_key = stable_states[i];
    }
    
    while (!osal_kthread_should_stop()) {
        uint32_t current_time = hfsys_get_time();
        
        // 扫描所有按键
        for (int i = 0; i < SWITCH_COUNT; i++) {
            bool new_raw_state = get_key_input(i);
            // if (i == 1) {
            //     e_printf("[KEY] 按键%d 状态: %d\r\n", i + 1, new_raw_state);
            // }
            // 如果原始状态发生变化，重置防抖计时
            if (new_raw_state != raw_states[i]) {
                raw_states[i] = new_raw_state;
                last_change_time[i] = current_time;
            }
            
            // 检查防抖时间是否达到
            if (current_time - last_change_time[i] >= KEY_DEBOUNCE_MS) {
                // 状态已稳定，可以处理
                if (raw_states[i] != stable_states[i]) {
                    // 稳定状态发生变化
                    bool old_stable = stable_states[i];
                    bool new_stable = raw_states[i];
                    stable_states[i] = new_stable;
                    
                    if (new_stable == KEY_PRESSED) {
                        // 按键按下
                        press_start_time[i] = current_time;
                        is_long_press_handled[i] = false;
                        e_printf("[KEY] 按键%d 按下\r\n", i + 1);
                    } else {
                        // 按键松开
                        uint32_t press_duration = current_time - press_start_time[i];
                        
                        if (press_duration >= KEY_LONG_PRESS_MS && !is_long_press_handled[i]) {
                            // 长按事件（如果在按下期间没有处理）
                            handle_key_long_press(i);
                            is_long_press_handled[i] = true;
                        } else if (press_duration < KEY_LONG_PRESS_MS) {
                            // 短按事件
                            handle_key_press(i);
                        }
                        
                        e_printf("[KEY] 按键%d 松开 (持续时间: %ums)\r\n", i + 1, press_duration);
                    }
                    
                    // 更新全局状态
                    g_key_states[i] = new_stable;
                    g_runtime_state.switches[i].physical_key = new_stable;
                }
            }
            
            // 检查长按（在按下期间持续监测）
            if (stable_states[i] == KEY_PRESSED && !is_long_press_handled[i]) {
                uint32_t press_duration = current_time - press_start_time[i];
                if (press_duration >= KEY_LONG_PRESS_MS) {
                    handle_key_long_press(i);
                    is_long_press_handled[i] = true;
                }
            }
        }
        
        // 休眠一段时间再继续扫描
        osal_msleep(10);  // 10ms扫描间隔，提高响应性
    }
    
    e_printf("[KEY] 按键扫描任务退出\r\n");
    return 0;
}

// 处理按键短按事件
void handle_key_press(int key_id) {
    if (key_id < 0 || key_id >= SWITCH_COUNT) {
        return;
    }
    
    e_printf("[KEY] 处理按键%d 短按事件\r\n", key_id + 1);
    
    // 在配网模式下，忽略短按事件
    if (g_runtime_state.mode == MODE_CONFIG) {
        e_printf("[KEY] 配网模式下，忽略短按事件\r\n");
        return;
    }
    
    // 在产测模式下，忽略短按事件
    if (g_runtime_state.mode == MODE_FACTORY_TEST) {
        e_printf("[KEY] 产测模式下，忽略短按事件\r\n");
        return;
    }
    
    // 正常模式下切换开关状态
    bool current_state = g_persistent_state.switches[key_id].switch_on;
    update_switch_state(key_id, !current_state);
}

// 处理按键长按事件
void handle_key_long_press(int key_id) {
    if (key_id < 0 || key_id >= SWITCH_COUNT) {
        return;
    }
    
    e_printf("[KEY] 处理按键%d 长按事件\r\n", key_id + 1);
    
    // 只有第一个按键支持长按进入配网模式
    if (key_id == 0) {
        e_printf("[KEY] 长按第一个按键，检查配网条件\r\n");
        
        // 只有在正常模式下且设备未绑定时才能进入配网模式
        if (g_runtime_state.mode == MODE_NORMAL && !g_persistent_state.is_bound) {
            extern int g_config_key_id;
            g_config_key_id = key_id;  // 设置触发配网的按键ID
            enter_config_mode();
        } else {
            if (g_persistent_state.is_bound) {
                e_printf("[KEY] 设备已绑定，不能进入配网模式\r\n");
            } else {
                e_printf("[KEY] 非正常模式，不能进入配网模式\r\n");
            }
        }
    } else {
        e_printf("[KEY] 非第一个按键的长按，忽略\r\n");
    }
}

//====================== 初始化函数 ======================

int key_system_init(void) {
    int ret = ERRCODE_SUCC;
    
    // 初始化按键状态
    for (int i = 0; i < SWITCH_COUNT; i++) {
        g_key_states[i] = get_key_input(i);
        g_runtime_state.switches[i].physical_key = g_key_states[i];
    }
    
    // 创建按键扫描任务
    g_key_scan_task_handle = osal_kthread_create((osal_kthread_handler)key_scan_task, 
                                                NULL, 
                                                "key_scan", 
                                                TASK_STACK_SIZE);
    if (!g_key_scan_task_handle) {
        e_printf("[KEY] 创建按键扫描任务失败\r\n");
        return HF_FAIL;
    }
    
    // 设置任务优先级
    ret = osal_kthread_set_priority(g_key_scan_task_handle, TASK_PRIORITY_HIGH);
    if (ret != 0) {
        e_printf("[KEY] 设置按键扫描任务优先级失败: %d\r\n", ret);
    }
    
    e_printf("[KEY] 按键系统初始化完成\r\n");
    return HF_SUCCESS;
}

// 快速上报单个开关状态(兼容旧接口，内部使用异步上报)
void fast_report_switch(int switch_id) {
    fast_report_switch_async(switch_id);
}

// 快速上报总开关状态(兼容旧接口，内部使用异步上报)
void fast_report_master_switch(void) {
    fast_report_master_switch_async();
}