#include "hilink.h"
#include "securec.h"
#include "hilink_sal_defines.h"
#include "driver/pinctrl.h"
#include "driver/gpio.h"
#include "platform_core_rom.h"
#include "switch_panel.h"

//====================== 全局变量 ======================
device_persistent_state_t g_persistent_state = {
    .is_first_boot = true,
    .is_bound = false,
    .master_switch = false,
    .panel_led = true,
    .switches = {
        {.switch_on = false, .led_state = false},
        {.switch_on = false, .led_state = false},
        {.switch_on = false, .led_state = false},
        {.switch_on = false, .led_state = false}
    },
    .magic = DEVICE_DATA_MAGIC,
    .version = DEVICE_DATA_VERSION
};

device_runtime_state_t g_runtime_state = {
    .mode = MODE_NORMAL,
    .ble_mode_enabled = false,
    .config_start_time = 0,
    .config_key_id = -1,
    .config_led_blink_state = false,
    .factory_test_running = false,
    .last_save_time = 0,
    .switches = {
        {.physical_key = true, .press_time = 0, .debounce_flag = false, .long_press_handled = false},
        {.physical_key = true, .press_time = 0, .debounce_flag = false, .long_press_handled = false},
        {.physical_key = true, .press_time = 0, .debounce_flag = false, .long_press_handled = false},
        {.physical_key = true, .press_time = 0, .debounce_flag = false, .long_press_handled = false}
    }
};

timer_handle_t g_key_debounce_timer[SWITCH_COUNT] = {0};
// 配网定时器变量已移除，配网逻辑简化
osal_task *g_key_scan_task_handle = NULL;
osal_task *g_config_task_handle = NULL;
osal_task *g_save_task_handle = NULL;  // 异步保存任务句柄
osal_task *g_report_task_handle = NULL;  // 异步上报任务句柄

static bool g_initialized = false;
static uint32_t g_config_start_time = 0;
static bool g_config_led_state = false;

//====================== 异步保存系统 ======================
#include "osal_mutex.h"
#include "osal_semaphore.h"

static osal_mutex g_save_mutex;                 // 保护持久化状态的互斥锁
static osal_semaphore g_save_semaphore;         // 触发保存的信号量
static bool g_save_system_running = false;      // 异步保存系统是否运行中
static bool g_save_mutex_initialized = false;   // 互斥锁是否已初始化
static bool g_save_sem_initialized = false;     // 信号量是否已初始化

//====================== 异步上报系统实现 ======================
#include "osal_semaphore.h"
#include "osal_mutex.h"

static osal_semaphore g_report_semaphore;       // 触发上报的信号量
static osal_mutex g_report_mutex;               // 保护上报掩码的互斥锁
static volatile uint8_t g_report_mask = 0;      // 待上报的服务ID位掩码
static bool g_report_system_running = false;    // 异步上报系统是否运行中
static bool g_report_sem_initialized = false;   // 信号量是否已初始化
static bool g_report_mutex_initialized = false; // 互斥锁是否已初始化

// 异步上报任务
int async_report_task(void *arg) {
    (void)arg;
    
    e_printf("异步上报任务启动\r\n");
    
    // 引用外部的fast_report函数
    extern int fast_report(const char* svc_id);
    
    while (g_report_system_running && !osal_kthread_should_stop()) {
        // 等待上报信号量
        if (osal_sem_down(&g_report_semaphore) == OSAL_SUCCESS) {
            if (!g_report_system_running) {
                break; // 系统关闭
            }
            
            // 获取当前的上报掩码（使用互斥锁保护）
            uint8_t current_mask = 0;
            if (g_report_mutex_initialized && osal_mutex_lock(&g_report_mutex) == OSAL_SUCCESS) {
                current_mask = g_report_mask;
                g_report_mask = 0;  // 清零掩码
                osal_mutex_unlock(&g_report_mutex);
            } else {
                current_mask = g_report_mask;
                g_report_mask = 0;
            }
            
            if (current_mask == 0) {
                continue; // 无上报任务
            }
            
            e_printf("开始异步上报，掩码: 0x%02X\r\n", current_mask);
            
            // 上报总开关
            if (current_mask & REPORT_SWITCH_MASK) {
                fast_report("switch");
                e_printf("上报总开关状态\r\n");
                osal_msleep(50); // 防止上报过快
            }
            
            // 上报各个子开关
            for (int i = 0; i < SWITCH_COUNT; i++) {
                uint8_t switch_mask = (REPORT_SWITCH1_MASK << i);
                if (current_mask & switch_mask) {
                    char svc_id[16] = {0};
                    snprintf(svc_id, sizeof(svc_id), "switch%d", i + 1);
                    fast_report(svc_id);
                    e_printf("上报开关%d状态\r\n", i + 1);
                    osal_msleep(50); // 防止上报过快
                }
            }
            
            e_printf("异步上报完成\r\n");
        }
    }
    
    e_printf("异步上报任务退出\r\n");
    return 0;
}

// 初始化异步上报系统
int report_system_init(void) {
    // 初始化互斥锁
    if (osal_mutex_init(&g_report_mutex) != OSAL_SUCCESS) {
        e_printf("初始化上报互斥锁失败\r\n");
        return HF_FAIL;
    }
    g_report_mutex_initialized = true;
    
    // 初始化信号量（初始值为0）
    if (osal_sem_init(&g_report_semaphore, 0) != OSAL_SUCCESS) {
        e_printf("初始化上报信号量失败\r\n");
        osal_mutex_destroy(&g_report_mutex);
        g_report_mutex_initialized = false;
        return HF_FAIL;
    }
    g_report_sem_initialized = true;
    
    // 启动异步上报任务
    g_report_system_running = true;
    g_report_task_handle = osal_kthread_create((osal_kthread_handler)async_report_task,
                                             NULL,
                                             "report_task",
                                             TASK_STACK_SIZE);
    if (g_report_task_handle == NULL) {
        e_printf("创建异步上报任务失败\r\n");
        g_report_system_running = false;
        if (g_report_sem_initialized) {
            osal_sem_destroy(&g_report_semaphore);
            g_report_sem_initialized = false;
        }
        if (g_report_mutex_initialized) {
            osal_mutex_destroy(&g_report_mutex);
            g_report_mutex_initialized = false;
        }
        return HF_FAIL;
    }
    
    // 设置任务优先级为低优先级，避免影响实时性
    osal_kthread_set_priority(g_report_task_handle, TASK_PRIORITY_LOW);
    
    e_printf("异步上报系统初始化成功\r\n");
    return HF_SUCCESS;
}

// 清理异步上报系统
void report_system_deinit(void) {
    e_printf("清理异步上报系统\r\n");
    
    // 停止上报系统
    g_report_system_running = false;
    
    // 唤醒上报任务使其退出
    if (g_report_sem_initialized) {
        osal_sem_up(&g_report_semaphore);
    }
    
    // 等待任务退出
    if (g_report_task_handle) {
        osal_kthread_destroy(g_report_task_handle, 1);
        g_report_task_handle = NULL;
    }
    
    // 清理同步原语
    if (g_report_sem_initialized) {
        osal_sem_destroy(&g_report_semaphore);
        g_report_sem_initialized = false;
    }
    
    if (g_report_mutex_initialized) {
        osal_mutex_destroy(&g_report_mutex);
        g_report_mutex_initialized = false;
    }
    
    // 清零上报掩码
    g_report_mask = 0;
    
    e_printf("异步上报系统清理完成\r\n");
}

// 触发异步上报
void trigger_async_report(uint8_t report_mask) {
    if (!g_report_system_running || !g_report_sem_initialized) {
        e_printf("异步上报系统未初始化，使用同步上报\r\n");
        // 如果异步系统未初始化，回退到同步上报
        extern int fast_report(const char* svc_id);
        if (report_mask & REPORT_SWITCH_MASK) {
            fast_report("switch");
        }
        for (int i = 0; i < SWITCH_COUNT; i++) {
            if (report_mask & (REPORT_SWITCH1_MASK << i)) {
                char svc_id[16] = {0};
                snprintf(svc_id, sizeof(svc_id), "switch%d", i + 1);
                fast_report(svc_id);
            }
        }
        return;
    }
    
    // 使用互斥锁保护更新上报掩码
    bool should_signal = false;
    if (g_report_mutex_initialized && osal_mutex_lock(&g_report_mutex) == OSAL_SUCCESS) {
        uint8_t old_mask = g_report_mask;
        g_report_mask |= report_mask;
        should_signal = (old_mask == 0);
        osal_mutex_unlock(&g_report_mutex);
    } else {
        uint8_t old_mask = g_report_mask;
        g_report_mask |= report_mask;
        should_signal = (old_mask == 0);
    }
    
    // 如果是新的上报请求，唤醒上报任务
    if (should_signal) {
        osal_sem_up(&g_report_semaphore);
        e_printf("已触发异步上报，掩码: 0x%02X\r\n", report_mask);
    } else {
        e_printf("合并上报请求，掩码: 0x%02X\r\n", g_report_mask);
    }
}

// 异步上报单个开关
void fast_report_switch_async(int switch_id) {
    if (switch_id < 0 || switch_id >= SWITCH_COUNT) {
        return;
    }
    
    uint8_t switch_mask = (REPORT_SWITCH1_MASK << switch_id);
    trigger_async_report(switch_mask);
}

// 异步上报总开关
void fast_report_master_switch_async(void) {
    trigger_async_report(REPORT_SWITCH_MASK);
}

// 异步上报所有开关
void fast_report_all_switches_async(void) {
    trigger_async_report(REPORT_ALL_MASK);
}

//====================== 存储管理函数 ======================

// 计算数据校验码

// 计算校验和
static uint8_t calculate_checksum(const uint8_t* data, int len)
{
    uint8_t sum = 0;
    for (int i = 0; i < len; i++) {
        sum ^= data[i];
    }
    return sum;
}

// 从指定地址读取设备数据
static bool read_device_data_from_addr(uint32_t addr, uint8_t* data, uint32_t len)
{
    int total_size = sizeof(device_data_t) + len;
    device_data_t *data_all = malloc(total_size);
    uint8_t checksum;
    
    if (data_all == NULL) {
        e_printf("内存分配失败\r\n");
        return false;
    }
    
    int ret = hfuflash_read(addr, (char*)data_all, total_size);
    if (ret != total_size) {
        e_printf("从地址0x%x读取设备数据失败，实际读取长度：%d\r\n", addr, ret);
        goto lab_err;
    }

    checksum = calculate_checksum(data_all->data, data_all->data_len);
    if (checksum != data_all->checksum) {
        e_printf("check sum error:%x\n", data_all->version);
        goto lab_err;
    }

    if (data_all->magic != DEVICE_DATA_MAGIC) {
        e_printf("magic error:%x\n", data_all->magic);
        goto lab_err;
    }
    
    if (data_all->version > DEVICE_DATA_VERSION) {
        e_printf("versoin missmatch:%x\n", data_all->version);
        goto lab_err;
    }
    memcpy_s(data, len, data_all->data, data_all->data_len);
    free(data_all);
    return true;

lab_err:
    free(data_all);
    return false;
}

// 写入数据到指定地址
static bool write_device_data_to_addr(uint32_t addr, uint8_t* data, uint32_t len)
{
    uint32_t ret = hfuflash_erase_page(addr, 1);
    if (ret != HF_SUCCESS) {
        e_printf("擦除地址0x%x的Flash页失败，错误码：%d\r\n", addr, ret);
        return false;
    }
    uint32_t total_size = sizeof(device_data_t) + len;
    device_data_t *data_all = malloc(total_size);
    if (data_all == NULL) {
        e_printf("内存分配失败\r\n");
        return false;
    }
    memset_s(data_all, total_size, 0, total_size);
    data_all->data_len = len;
    memcpy_s(data_all->data, len, data, len);
    data_all->checksum = calculate_checksum(data_all->data, data_all->data_len);
    data_all->magic = DEVICE_DATA_MAGIC;
    data_all->version = DEVICE_DATA_VERSION;
    ret = hfuflash_write(addr, (char*)data_all, total_size);
    if (ret != total_size) {
        e_printf("写入数据到地址0x%x失败，实际写入长度：%d\r\n", addr, ret);
        free(data_all);
        return false;
    }
    free(data_all);
    return true;
}


// 从 Flash 加载持久化状态
int load_persistent_state(void) {
    int ret = 0;
    device_persistent_state_t state;
    bool valid = false;
    
    // 尝试读取主数据区
    valid = read_device_data_from_addr(DEVICE_DATA_FLASH_ADDR, (uint8_t*)&state, sizeof(state));
    
    // 如果主数据区无效，尝试读取备份区
    if (!valid) {
        valid = read_device_data_from_addr(DEVICE_DATA_BACKUP_ADDR, (uint8_t*)&state, sizeof(state));
    }
    
    // 两个存储区都失败，使用默认状态  
    if (!valid) {
        e_printf("存储区数据损坏，使用默认状态\r\n");
        reset_persistent_state();
        save_persistent_state();
        return 0;
    }
    
    // 更新持久化状态
    e_printf("持久化状态恢复:\r\n");
    e_printf("首次启动: %d => %d\r\n", g_persistent_state.is_first_boot, state.is_first_boot);
    e_printf("配网状态: %d => %d\r\n", g_persistent_state.is_bound, state.is_bound);
    e_printf("总开关: %d => %d\r\n", g_persistent_state.master_switch, state.master_switch);
    e_printf("面板背光: %d => %d\r\n", g_persistent_state.panel_led, state.panel_led);
    
    for (int i = 0; i < SWITCH_COUNT; i++) {
        e_printf("开关%d(%s): %d => %d, LED%d: %d => %d\r\n", 
                i + 1, state.switches[i].name,
                g_persistent_state.switches[i].switch_on, state.switches[i].switch_on,
                i + 1, g_persistent_state.switches[i].led_state, state.switches[i].led_state);
    }
    
    memcpy(&g_persistent_state, &state, sizeof(device_persistent_state_t));

    return 0;
}

//====================== 异步保存系统实现 ======================

// 异步保存任务
static int save_data_task(void *arg) {
    (void)arg;
    
    e_printf("异步保存任务启动\r\n");
    
    while (g_save_system_running && !osal_kthread_should_stop()) {
        // 等待保存信号量
        if (osal_sem_down(&g_save_semaphore) == OSAL_SUCCESS) {
            if (!g_save_system_running) {
                break; // 系统关闭
            }
            
            // 执行实际的同步保存操作
            int ret = save_persistent_state_sync();
            if (ret != HF_SUCCESS) {
                e_printf("异步保存失败: %d\r\n", ret);
            } else {
                e_printf("异步保存成功\r\n");
            }
        }
    }
    
    e_printf("异步保存任务退出\r\n");
    return 0;
}

// 初始化异步保存系统
int save_system_init(void) {
    int ret = HF_SUCCESS;
    
    // 初始化互斥锁
    if (osal_mutex_init(&g_save_mutex) != OSAL_SUCCESS) {
        e_printf("初始化保存互斥锁失败\r\n");
        return HF_FAIL;
    }
    g_save_mutex_initialized = true;
    
    // 初始化信号量（初始值为0）
    if (osal_sem_init(&g_save_semaphore, 0) != OSAL_SUCCESS) {
        e_printf("初始化保存信号量失败\r\n");
        osal_mutex_destroy(&g_save_mutex);
        g_save_mutex_initialized = false;
        return HF_FAIL;
    }
    g_save_sem_initialized = true;
    
    // 启动异步保存任务
    g_save_system_running = true;
    g_save_task_handle = osal_kthread_create((osal_kthread_handler)save_data_task,
                                           NULL,
                                           "save_data_task",
                                           TASK_STACK_SIZE);
    if (g_save_task_handle == NULL) {
        e_printf("创建异步保存任务失败\r\n");
        g_save_system_running = false;
        if (g_save_sem_initialized) {
            osal_sem_destroy(&g_save_semaphore);
            g_save_sem_initialized = false;
        }
        if (g_save_mutex_initialized) {
            osal_mutex_destroy(&g_save_mutex);
            g_save_mutex_initialized = false;
        }
        return HF_FAIL;
    }
    
    // 设置任务优先级为低优先级，避免影响实时性
    osal_kthread_set_priority(g_save_task_handle, TASK_PRIORITY_LOW);
    
    e_printf("异步保存系统初始化成功\r\n");
    return HF_SUCCESS;
}

// 清理异步保存系统
void save_system_deinit(void) {
    e_printf("清理异步保存系统\r\n");
    
    // 停止保存系统
    g_save_system_running = false;
    
    // 唤醒保存任务使其退出
    if (g_save_sem_initialized) {
        osal_sem_up(&g_save_semaphore);
    }
    
    // 等待任务退出
    if (g_save_task_handle) {
        osal_kthread_destroy(g_save_task_handle, 1);
        g_save_task_handle = NULL;
    }
    
    // 清理同步原语
    if (g_save_sem_initialized) {
        osal_sem_destroy(&g_save_semaphore);
        g_save_sem_initialized = false;
    }
    
    if (g_save_mutex_initialized) {
        osal_mutex_destroy(&g_save_mutex);
        g_save_mutex_initialized = false;
    }
    
    e_printf("异步保存系统清理完成\r\n");
}

// 异步保存持久化状态（触发保存任务）
int save_persistent_state(void) {
    if (!g_save_system_running || !g_save_sem_initialized) {
        e_printf("异步保存系统未初始化，使用同步保存\r\n");
        return save_persistent_state_sync();
    }
    
    // 发送保存信号
    osal_sem_up(&g_save_semaphore);
    
    e_printf("已触发异步保存\r\n");
    return HF_SUCCESS;
}

// 同步保存持久化状态到 Flash（实际的Flash写入操作）
int save_persistent_state_sync(void) {
    int ret = 0;
    static device_persistent_state_t state = {};
    bool need_save = false;
    
    // 使用互斥锁保护状态访问
    if (g_save_mutex_initialized && osal_mutex_lock(&g_save_mutex) != OSAL_SUCCESS) {
        e_printf("获取保存互斥锁失败\r\n");
        return HF_FAIL;
    }
    
    // 检查是否有变化，避免不必要的Flash写入
    if (memcmp(&state, &g_persistent_state, sizeof(device_persistent_state_t)) != 0) {
        need_save = true;
    }
    
    if (!need_save) {
        if (g_save_mutex_initialized) {
            osal_mutex_unlock(&g_save_mutex);
        }
        e_printf("持久化状态未变化，跳过保存\r\n");
        return HF_SUCCESS;
    }
    
    // 准备数据
    memcpy(&state, &g_persistent_state, sizeof(device_persistent_state_t));
    
    // 释放互斥锁，避免在Flash写入过程中长时间锁定
    if (g_save_mutex_initialized) {
        osal_mutex_unlock(&g_save_mutex);
    }
    
    // 保存到主存储区
    bool ret_main = write_device_data_to_addr(DEVICE_DATA_FLASH_ADDR, (uint8_t*)&state, sizeof(state));
    if (!ret_main) {
        e_printf("写入主存储区失败\r\n");
        return HF_FAIL;
    }
    
    // 保存到备份区
    bool ret_backup = write_device_data_to_addr(DEVICE_DATA_BACKUP_ADDR, (uint8_t*)&state, sizeof(state));
    if (!ret_backup) {
        e_printf("写入备份区失败\r\n");
    }
    
    g_runtime_state.last_save_time = hfsys_get_time();
    e_printf("持久化状态保存完成\r\n");
    return HF_SUCCESS;
}

// 重置持久化状态为默认值
void reset_persistent_state(void) {
    // 设置默认状态
    g_persistent_state.master_switch = false;     // 总开关关闭
    g_persistent_state.panel_led = true;          // 面板背光开启
    g_persistent_state.is_bound = false;          // 设备未绑定
    g_persistent_state.is_first_boot = true;     // 标记为首次启动
    
    // 所有开关默认关闭，所有LED默认黄灯，初始化默认名字
    for (int i = 0; i < SWITCH_COUNT; i++) {
        g_persistent_state.switches[i].switch_on = false;   // 开关关闭
        g_persistent_state.switches[i].led_state = false;   // LED黄灯
        snprintf(g_persistent_state.switches[i].name, SWITCH_NAME_MAX_LEN, "开关%d", i + 1);  // 默认名字
    }
    
    g_persistent_state.magic = DEVICE_DATA_MAGIC;
    g_persistent_state.version = DEVICE_DATA_VERSION;
    
    e_printf("[STATE] 持久化状态已重置为默认值\r\n");
}

// 初始化运行时状态
void init_runtime_state(void) {
    g_runtime_state.mode = MODE_NORMAL;
    g_runtime_state.ble_mode_enabled = false;
    g_runtime_state.config_start_time = 0;
    g_runtime_state.config_key_id = -1;
    g_runtime_state.config_led_blink_state = false;
    g_runtime_state.factory_test_running = false;
    g_runtime_state.last_save_time = 0;
    
    // 初始化所有开关的运行时状态
    for (int i = 0; i < SWITCH_COUNT; i++) {
        g_runtime_state.switches[i].physical_key = true;      // 按键松开
        g_runtime_state.switches[i].press_time = 0;
        g_runtime_state.switches[i].debounce_flag = false;
        g_runtime_state.switches[i].long_press_handled = false;
    }
    
    e_printf("[STATE] 运行时状态已初始化\r\n");
}

//====================== GPIO配置数据结构 ======================
typedef struct {
    pin_t pin;
    pin_mode_t mode;
    pin_pull_t pull;
    pin_drive_strength_t ds;
    gpio_direction_t direction;
    const char* name;
} gpio_config_t;

// GPIO初始化配置表
static const gpio_config_t gpio_configs[] = {
    // 开关控制 - 输出
    {SWITCH1_GPIO, PIN_MODE_1, PIN_PULL_TYPE_DOWN, PIN_DS_7, GPIO_DIRECTION_OUTPUT, "SWITCH1"},
    {SWITCH2_GPIO, PIN_MODE_1, PIN_PULL_TYPE_DOWN, PIN_DS_7, GPIO_DIRECTION_OUTPUT, "SWITCH2"},
    {SWITCH3_GPIO, PIN_MODE_1, PIN_PULL_TYPE_DOWN, PIN_DS_7, GPIO_DIRECTION_OUTPUT, "SWITCH3"},
    {SWITCH4_GPIO, PIN_MODE_1, PIN_PULL_TYPE_DOWN, PIN_DS_7, GPIO_DIRECTION_OUTPUT, "SWITCH4"},
    
    // 物理按键 - 输入
    {KEY1_GPIO, PIN_MODE_0, PIN_PULL_TYPE_UP, PIN_DS_7, GPIO_DIRECTION_INPUT, "KEY1"},
    {KEY2_GPIO, PIN_MODE_0, PIN_PULL_TYPE_UP, PIN_DS_7, GPIO_DIRECTION_INPUT, "KEY2"},
    {KEY3_GPIO, PIN_MODE_0, PIN_PULL_TYPE_UP, PIN_DS_7, GPIO_DIRECTION_INPUT, "KEY3"},
    {KEY4_GPIO, PIN_MODE_0, PIN_PULL_TYPE_UP, PIN_DS_7, GPIO_DIRECTION_INPUT, "KEY4"},
    
    // 面板背光 - 输出
    {PANEL_LED_GPIO, PIN_MODE_1, PIN_PULL_TYPE_DOWN, PIN_DS_7, GPIO_DIRECTION_OUTPUT, "PANEL_LED"},

    // LED指示灯 - 输出
    {LED1_GPIO, PIN_MODE_1, PIN_PULL_TYPE_DOWN, PIN_DS_7, GPIO_DIRECTION_OUTPUT, "LED1"},
    {LED2_GPIO, PIN_MODE_1, PIN_PULL_TYPE_DOWN, PIN_DS_7, GPIO_DIRECTION_OUTPUT, "LED2"},
    {LED3_GPIO, PIN_MODE_1, PIN_PULL_TYPE_DOWN, PIN_DS_7, GPIO_DIRECTION_OUTPUT, "LED3"},
    {LED4_GPIO, PIN_MODE_1, PIN_PULL_TYPE_DOWN, PIN_DS_7, GPIO_DIRECTION_OUTPUT, "LED4"},
    
};

#define GPIO_CONFIG_COUNT (sizeof(gpio_configs) / sizeof(gpio_config_t))

//====================== 硬件控制函数 ======================

// GPIO 初始化
int switch_panel_gpio_init(void) {
    int ret = HF_SUCCESS;
    
    // 初始化pinctrl和GPIO
    uapi_pin_init();
    uapi_gpio_init();
    
    // 使用循环配置所有GPIO
    for (uint32_t i = 0; i < GPIO_CONFIG_COUNT; i++) {
        const gpio_config_t* config = &gpio_configs[i];
        
        // 设置为GPIO模式
        ret = uapi_pin_set_mode(config->pin, config->mode);
        if (ret != 0) {
            e_printf("[GPIO] %s pinctrl初始化失败: %d\r\n", config->name, ret);
            return HF_FAIL;
        }
        ret = uapi_pin_set_pull(config->pin, config->pull);
        if (ret != 0) {
            e_printf("[GPIO] %s pinctrl初始化失败: %d\r\n", config->name, ret);
            return HF_FAIL;
        }
        ret = uapi_pin_set_ds(config->pin, config->ds);
        if (ret != 0) {
            e_printf("[GPIO] %s pinctrl初始化失败: %d\r\n", config->name, ret);
            return HF_FAIL;
        }
        
        // 设置GPIO方向
        ret = uapi_gpio_set_dir(config->pin, config->direction);
        if (ret != 0) {
            e_printf("[GPIO] %s 设置方向失败: %d\r\n", config->name, ret);
            return HF_FAIL;
        }
        
        e_printf("[GPIO] %s 初始化完成 (方向: %s)\r\n", 
                config->name, 
                config->direction == GPIO_DIRECTION_OUTPUT ? "输出" : "输入");
    }
    
    e_printf("[GPIO] 所有GPIO初始化完成 (共%d个)\r\n", GPIO_CONFIG_COUNT);
    return HF_SUCCESS;
}

// 设置开关输出状态
void set_switch_output(int switch_id, bool state) {
    if (switch_id < 0 || switch_id >= SWITCH_COUNT) {
        e_printf("[HW] 无效的开关ID: %d\r\n", switch_id);
        return;
    }
    
    pin_t gpio_pin;
    switch (switch_id) {
        case 0: gpio_pin = SWITCH1_GPIO; break;  // SWITCH1
        case 1: gpio_pin = SWITCH2_GPIO; break;  // SWITCH2
        case 2: gpio_pin = SWITCH3_GPIO; break;  // SWITCH3
        case 3: gpio_pin = SWITCH4_GPIO; break;  // SWITCH4
        default: return;
    }
    
    // 设置输出状态
    gpio_level_t level = state ? GPIO_LEVEL_HIGH : GPIO_LEVEL_LOW;
    uapi_gpio_set_val(gpio_pin, level);
    e_printf("[HW] 开关%d 状态: %s\r\n", switch_id + 1, state ? "开" : "关");
}

// 设置 LED 指示灯状态
void set_led_output(int led_id, led_state_t state) {
    if (led_id < 0 || led_id >= SWITCH_COUNT) {
        e_printf("[HW] 无效的LED ID: %d\r\n", led_id);
        return;
    }
    
    pin_t gpio_pin;
    switch (led_id) {
        case 0: gpio_pin = LED1_GPIO; break;  // LED1
        case 1: gpio_pin = LED2_GPIO; break;  // LED2
        case 2: gpio_pin = LED3_GPIO; break;  // LED3
        case 3: gpio_pin = LED4_GPIO; break;  // LED4
        default: return;
    }
    
    // 设置LED状态 (高电平=白灯, 低电平=黄灯)
    gpio_level_t level = (state == LED_WHITE) ? GPIO_LEVEL_HIGH : GPIO_LEVEL_LOW;
    uapi_gpio_set_val(gpio_pin, level);
}

// 设置面板背光状态
void set_panel_led(panel_led_state_t state) {
    // 设置面板背光状态 (高电平=亮黄灯, 低电平=灭灯)
    gpio_level_t level = (state == PANEL_LED_ON) ? GPIO_LEVEL_HIGH : GPIO_LEVEL_LOW;
    uapi_gpio_set_val(PANEL_LED_GPIO, level);
}

// 获取按键输入状态
bool get_key_input(int key_id) {
    if (key_id < 0 || key_id >= SWITCH_COUNT) {
        return true;  // 默认松开状态
    }
    
    pin_t gpio_pin;
    switch (key_id) {
        case 0: gpio_pin = KEY1_GPIO; break;  // KEY1
        case 1: gpio_pin = KEY2_GPIO; break;  // KEY2
        case 2: gpio_pin = KEY3_GPIO; break;  // KEY3
        case 3: gpio_pin = KEY4_GPIO; break;  // KEY4
        default: return true;
    }
    
    // 获取输入状态 (低电平=按下, 高电平=松开)
    gpio_level_t level = uapi_gpio_get_val(gpio_pin);
    return (level == GPIO_LEVEL_HIGH);
}

//====================== 设备状态同步函数 ======================

// 同步硬件状态与持久化状态
void sync_hardware_state(void) {
    // 同步所有开关状态
    for (int i = 0; i < SWITCH_COUNT; i++) {
        set_switch_output(i, g_persistent_state.switches[i].switch_on);
        set_led_output(i, g_persistent_state.switches[i].led_state ? LED_WHITE : LED_YELLOW);
    }
    
    // 同步面板背光状态
    set_panel_led(g_persistent_state.panel_led ? PANEL_LED_ON : PANEL_LED_OFF);
    
    e_printf("[STATE] 硬件状态已同步\r\n");
}

//====================== 主程序函数 ======================

int switch_panel_main(void) {
    int ret = HF_SUCCESS;
    
    if (g_initialized) {
        e_printf("开关面板已初始化\r\n");
        return HF_SUCCESS;
    }
    g_initialized = true;
    e_printf("开始初始化SORONTEK智能面板...\r\n");
        
    // 初始化运行时状态
    init_runtime_state();
    
    // 初始化异步保存系统
    ret = save_system_init();
    if (ret != HF_SUCCESS) {
        e_printf("异步保存系统初始化失败: %d\r\n", ret);
        return ret;
    }
    
    // 初始化异步上报系统
    ret = report_system_init();
    if (ret != HF_SUCCESS) {
        e_printf("异步上报系统初始化失败: %d\r\n", ret);
        // 上报系统失败不影响主流程，继续运行
    }
        
    // 加载持久化状态
    ret = load_persistent_state();
    if (ret != HF_SUCCESS) {
        e_printf("加载持久化状态失败: %d\r\n", ret);
        return ret;
    }
    
    // 检查是否首次启动
    bool first_boot = g_persistent_state.is_first_boot;
    
    e_printf("设备状态 - 首次启动: %s, 绑定状态: %s\r\n", 
           first_boot ? "是" : "否", 
           g_persistent_state.is_bound ? "已绑定" : "未绑定");
    // 初始化GPIO
    ret = switch_panel_gpio_init();
    if (ret != HF_SUCCESS) {
        e_printf("GPIO初始化失败: %d\r\n", ret);
        return ret;
    }
    if(key_system_init() != HF_SUCCESS) {
        e_printf("按键系统初始化失败: %d\r\n", ret);
        return ret;
    }
    
    // 同步硬件状态
    sync_hardware_state();
    
    // 保存状态更新
    save_persistent_state();
    
    // 配网逻辑：
    // 1. 如果设备未绑定 且 是第一次启动 -> 直接进入配网
    // 2. 如果设备未绑定 且 不是第一次启动 -> 等待按键触发配网
    if (!g_persistent_state.is_bound) {
        if (first_boot) {
            e_printf("首次启动且未绑定，直接进入配网模式\r\n");
            enter_config_mode();
        } else {
            e_printf("设备未绑定，等待按键触发配网\r\n");
            // 配网逻辑将在按键处理函数中实现
        }
    } else {
        e_printf("设备已绑定，正常运行\r\n");
        // 设备已绑定时禁用蓝牙模式，使用云端控制
        extern void switch_panel_ble_disable(void);
        switch_panel_ble_disable();
    }
    
    e_printf("SORONTEK智能面板初始化完成\r\n");
    print_device_state();
    
    return HF_SUCCESS;
}

//====================== 调试函数 ======================

void print_device_state(void) {
    e_printf("\r\n===== 设备状态信息 =====\r\n");
    e_printf("总开关: %s\r\n", g_persistent_state.master_switch ? "开" : "关");
    e_printf("面板背光: %s\r\n", g_persistent_state.panel_led ? "开" : "关");
    e_printf("绑定状态: %s\r\n", g_persistent_state.is_bound ? "已绑定" : "未绑定");
    e_printf("首次启动: %s\r\n", g_persistent_state.is_first_boot ? "是" : "否");
    e_printf("工作模式: %s\r\n", get_mode_string(g_runtime_state.mode));
    
    for (int i = 0; i < SWITCH_COUNT; i++) {
        e_printf("开关%d(%s): %s, LED: %s\r\n", 
               i + 1,
               g_persistent_state.switches[i].name,
               g_persistent_state.switches[i].switch_on ? "开" : "关",
               g_persistent_state.switches[i].led_state ? "白灯" : "黄灯");
    }
    e_printf("=======================\r\n\r\n");
}

const char* get_mode_string(system_mode_t mode) {
    switch (mode) {
        case MODE_NORMAL: return "正常模式";
        case MODE_CONFIG: return "配网模式";
        case MODE_FACTORY_TEST: return "产测模式";
        case MODE_UNBIND: return "解绑模式";
        default: return "未知模式";
    }
}

//====================== 开关名字操作函数 ======================

// 获取开关名字
const char* get_switch_name(int switch_id) {
    if (switch_id < 0 || switch_id >= SWITCH_COUNT) {
        return "";
    }
    return g_persistent_state.switches[switch_id].name;
}

// 设置开关名字
int set_switch_name(int switch_id, const char* name) {
    if (switch_id < 0 || switch_id >= SWITCH_COUNT || !name) {
        e_printf("无效的开关ID或名字参数: switch_id=%d\r\n", switch_id);
        return HF_FAIL;
    }
    
    // 使用互斥锁保护状态访问
    if (g_save_mutex_initialized && osal_mutex_lock(&g_save_mutex) != OSAL_SUCCESS) {
        e_printf("获取互斥锁失败\r\n");
        return HF_FAIL;
    }
    
    // 检查名字是否相同，相同则跳过保存
    if (strcmp(g_persistent_state.switches[switch_id].name, name) == 0) {
        if (g_save_mutex_initialized) {
            osal_mutex_unlock(&g_save_mutex);
        }
        e_printf("开关%d名字未变化，跳过保存: %s\r\n", switch_id + 1, name);
        return HF_SUCCESS;
    }
    
    // 复制名字，确保不会溢出
    int ret = strncpy_s(g_persistent_state.switches[switch_id].name, 
                       SWITCH_NAME_MAX_LEN, 
                       name, 
                       SWITCH_NAME_MAX_LEN - 1);
    
    if (ret != 0) {
        e_printf("开关%d名字复制失败: %d\r\n", switch_id + 1, ret);
        if (g_save_mutex_initialized) {
            osal_mutex_unlock(&g_save_mutex);
        }
        return HF_FAIL;
    }
    
    // 确保字符串以NULL结尾
    g_persistent_state.switches[switch_id].name[SWITCH_NAME_MAX_LEN - 1] = '\0';
    
    if (g_save_mutex_initialized) {
        osal_mutex_unlock(&g_save_mutex);
    }
    
    e_printf("开关%d名字已更新为: %s\r\n", 
           switch_id + 1, g_persistent_state.switches[switch_id].name);
    
    // 异步保存状态
    save_persistent_state();
    
    return HF_SUCCESS;
}

// 初始化默认开关名字
void init_default_switch_names(void) {
    for (int i = 0; i < SWITCH_COUNT; i++) {
        snprintf(g_persistent_state.switches[i].name, SWITCH_NAME_MAX_LEN, "开关%d", i + 1);
    }
    e_printf("默认开关名字已初始化\r\n");
}

//====================== 状态访问便利函数 ======================

// 获取开关状态
bool get_switch_state(int switch_id) {
    if (switch_id < 0 || switch_id >= SWITCH_COUNT) {
        return false;
    }
    return g_persistent_state.switches[switch_id].switch_on;
}

// 获取总开关状态
bool get_master_switch_state(void) {
    return g_persistent_state.master_switch;
}

// 获取设备绑定状态
bool is_device_bound(void) {
    return g_persistent_state.is_bound;
}

// 获取是否首次启动
bool is_first_boot(void) {
    return g_persistent_state.is_first_boot;
}

// 获取当前工作模式
system_mode_t get_device_mode(void) {
    return g_runtime_state.mode;
}

// 设置设备工作模式
void set_device_mode(system_mode_t mode) {
    if (g_runtime_state.mode != mode) {
        system_mode_t old_mode = g_runtime_state.mode;
        g_runtime_state.mode = mode;
        
        e_printf("[MODE] 设备模式切换: %s -> %s\r\n", 
               get_mode_string(old_mode), get_mode_string(mode));
    }
}